#!/usr/bin/env python
import core
import event
import omegle
import troll
import listener

from core import startTrolling, stopTrolling

# Test imports
if __name__ == '__main__':
    m = troll.MiddleMan()
    startTrolling()
